@echo off

setlocal

if not defined MAYA_BIN (
	for %%v in (2015,2016,2017,2018) do (
		if exist "C:\Program Files\Autodesk\Maya%%v\bin" set MAYA_BIN=C:\Program Files\Autodesk\Maya%%v\bin
		if exist "C:\Autodesk\Maya%%v\bin" set MAYA_BIN=C:\Autodesk\Maya%%v\bin
	)
)

if not defined MAYA_BIN (
	echo.
	echo Please set the MAYA_BIN variable to the bin directory of your Maya installation, like 'set MAYA_BIN=C:\Program Files\Autodesk\Maya2018\bin'
	exit /b 1
)

echo Using MAYA_BIN = %MAYA_BIN%

"%MAYA_BIN%\mayapy" -V >nul 2>&1

if errorlevel 1 (
	echo.
	echo Maya bin directory not found: '%MAYA_BIN%'
	exit /b 1
)

cd/d %~dp0

:: No need to create .pyc files, the difference in loading speed is negligible, they just clutter the directories
set PYTHONDONTWRITEBYTECODE=1

:: Disable performance tests by default. 0 = disabled, 1 = enabled, 2 = run once
if "%PerformanceTestsMode%"=="" set PerformanceTestsMode=0

:: Tests can be run from inside Maya by typing in the Python window:
::   import sys
::   sys.path.append( r'C:\ProgramData\Ephere\OrnatrixMaya\Test' )
::   import RunTests
::   RunTests.main()

"%MAYA_BIN%\mayapy" RunTests.py %1
::if "%1"=="" "%MAYA_BIN%\maya" -noAutoloadPlugins -command "python \"import sys; import os; sys.path.append( os.getcwd() ); import RunTests; exitCode = RunTests.main(); os._exit( exitCode )\";"
::if not "%1"=="" "%MAYA_BIN%\maya" -noAutoloadPlugins -command "python \"import sys; import os; sys.path.append( os.getcwd() ); import RunTests; exitCode = RunTests.main( name = '%1' ); os._exit( exitCode )\";"
