﻿import MayaTest
import pymel.core as pm
import pymel.core.datatypes as dt

AdoptExternalGuidesNodeName = 'AdoptExternalGuidesNode'
AnimationCacheNodeName = 'AnimationCacheNode'
BakedHairNodeName = 'BakedHairNode'
BakedGuidesNodeName = 'BakedGuidesNode'
BraidGuidesNodeName = 'BraidGuidesNode'
ChangeWidthNodeName = 'ChangeWidthNode'
ClumpNodeName = 'ClumpNode'
CurlNodeName = 'CurlNode'
CurvesFromStrandsNodeName = 'CurvesFromStrandsNode'
EditGuidesCommandName = 'OxEditGuides'
DetailNodeName = "DetailNode"
DynamicsNodeName = 'DynamicsNode'
EditGuidesShapeName = 'EditGuidesShape'
FrizzNodeName = 'FrizzNode'
GenerateStrandDataNodeName = 'GenerateStrandDataNode'
GravityNodeName = 'GravityNode'
GroundStrandsNodeName = 'GroundStrandsNode'
GuideClusterNodeName = 'GuideClusterNode'
GuidesFromCurvesNodeName = 'GuidesFromCurvesNode'
GuidesFromGuideMeshNodeName = 'GuidesFromGuideMeshNode'
GuidesFromHairNodeName = 'GuidesFromHairNode'
GuidesFromMeshNodeName = 'GuidesFromMeshNode'
GuidesShapeName = 'GuidesShape'
HairClusterNodeName = 'HairClusterNode'
HairFromGuidesNodeName = 'HairFromGuidesNode'
HairFromMeshStripsNodeName = 'HairFromMeshStripsNode'
HairShapeName = 'HairShape'
LengthNodeName = 'LengthNode'
MergeNodeName = 'MergeNode'
MeshFromStrandsNodeName = 'MeshFromStrandsNode'
MoovPhysicsNodeName = 'MoovPhysicsNode'
MoovPhysicsLocatorNodeName = 'MoovPhysicsLocatorNode'
PolyPlaneShapeName = 'polyPlane'
PushAwayFromSurfaceNodeName = 'PushAwayFromSurfaceNode'
RenderSettingsNodeName = 'RenderSettingsNode'
ResolveCollisionsNodeName = 'ResolveCollisionsNode'
RotateNodeName = 'RotateNode'
ScatterNodeName = 'ScatterNode'
ShellsNodeName = 'ShellsNode'
SurfaceCombNodeName = 'SurfaceCombNode'
SymmetryNodeName = 'SymmetryNode'
MultiplierNodeName = 'MultiplierNode'
NormalizeStrandsNodeName = 'NormalizeStrandsNode'
PropagationNodeName = 'PropagationNode'
TransferNormalsNodeName = 'TransferNormalsNode'

def GetNodeByType( nodeTypeName ):
	return pm.PyNode( pm.ls( type = nodeTypeName )[0] )

def CreateNode( typeName ):
	if typeName[-4:] == 'Node':
		return pm.createNode( typeName, name = typeName[0:-4] + '#' )
	else:
		return pm.createNode( typeName )


def CheckTipPositionsEqual( self, strandsShape, expectedPositions ):
	# Compare the tip positions of each guide, they should be the same as before
	for i in range( 0, pm.mel.OxGetStrandCount( strandsShape ) ):
		tipPosition = pm.mel.OxGetStrandPoint( strandsShape, i, 1 )
		self.assertLess( dt.Vector( tipPosition ).distanceTo( expectedPositions[i] ), 0.0001,
			'i: %(i)d, tipPosition: %(tipPosition)s, expected: %(expected)s' % { 'i': i, 'tipPosition': tipPosition, 'expected' : expectedPositions[i] } )

def AddHairToMesh( meshShape, hairRenderCount = 100, hairViewportCount = 10, rootGenerationMethod = 2, pointsPerStrandCount = 2, guideRootGenerationMethod = 4, guideCount = 10, lengthRandomness = 0, length = 50,
				 guideArea = 2 ):
	# Create nodes
	transformNode = meshShape.firstParent2()
	guidesFromMesh = CreateNode( GuidesFromMeshNodeName )
	guidesFromMesh.lengthRandomness.set( lengthRandomness )
	guidesFromMesh.length.set( length )
	guidesFromMesh.pointCount.set( pointsPerStrandCount )
	guidesFromMesh.distribution.set( guideRootGenerationMethod )
	guidesFromMesh.attr( 'count' ).set( guideCount )
	hairFromGuides = CreateNode( HairFromGuidesNodeName )
	hairShape = pm.createNode( HairShapeName, parent = transformNode )

	# Set parameters
	hairFromGuides.renderCount.set( hairRenderCount )
	hairFromGuides.viewportCountFraction.set( hairViewportCount / ( hairRenderCount * 1.0 ) )
	hairFromGuides.distribution.set( rootGenerationMethod )
	hairFromGuides.guideArea.set( guideArea )

	# Make connections
	meshShape.outMesh >> guidesFromMesh.inputMesh
	guidesFromMesh.outputGuides >> hairFromGuides.inputGuides
	guidesFromMesh.outputStack >> hairFromGuides.inputStack
	hairFromGuides.outputHair >> hairShape.inputHair
	meshShape.outMesh >> hairShape.distributionMesh
	meshShape.outMesh >> hairFromGuides.distributionMesh

	return hairShape

def AddGuidesToMesh( meshShape, guideCount = 10, rootGenerationMethod = 4, pointsPerStrandCount = 2, lengthRandomness = 0, length = 50, randomSeed = 1 ):
	# Create nodes
	transformNode = meshShape.firstParent2()
	guidesFromMesh = CreateNode( GuidesFromMeshNodeName )
	guidesShape = pm.createNode( HairShapeName, parent = transformNode )

	# Set parameters
	guidesFromMesh.attr( 'count' ).set( guideCount )
	guidesFromMesh.distribution.set( rootGenerationMethod )
	guidesFromMesh.pointCount.set( pointsPerStrandCount )
	guidesFromMesh.lengthRandomness.set( lengthRandomness )
	guidesFromMesh.length.set( length )
	guidesFromMesh.randomSeed.set( randomSeed )

	# Make connections
	meshShape.outMesh >> guidesFromMesh.inputMesh
	guidesFromMesh.outputGuides >> guidesShape.inputHair
	meshShape.outMesh >> guidesShape.distributionMesh

	return guidesShape

def AddGuidesToNewPlane( guideCount = 10, rootGenerationMethod = 4, pointsPerStrandCount = 2, planeSegmentCount = 1, planeSize = 1, length = 50, lengthRandomness = 0 ):
	plane = pm.polyPlane( sx = planeSegmentCount, sy = planeSegmentCount, width = planeSize, height = planeSize )
	pm.select( plane )
	meshShape = plane[0]

	return AddGuidesToMesh( meshShape, guideCount, rootGenerationMethod, pointsPerStrandCount, lengthRandomness, length )

def AddBakedHair():
	# Create nodes
	plane = pm.polyPlane( sx = 1, sy = 1, width = 10, height = 10 )
	pm.select( plane )
	meshShape = plane[0]
	transformNode = meshShape.firstParent2()

	bakedHairNode = CreateNode( BakedHairNodeName )
	hairShape = pm.createNode( HairShapeName, parent = transformNode )

	# Make connections
	meshShape.outMesh >> bakedHairNode.distributionMesh
	bakedHairNode.outputHair >> hairShape.inputHair
	meshShape.outMesh >> hairShape.distributionMesh

	return hairShape
	
def AddBakedGuides():
	# Create nodes
	plane = pm.polyPlane( sx = 1, sy = 1, width = 10, height = 10 )
	pm.select( plane )
	meshShape = plane[0]
	transformNode = meshShape.firstParent2()

	bakedGuidesNode = CreateNode( BakedGuidesNodeName )
	guidesShape = pm.createNode( GuidesShapeName, parent = transformNode )

	# Make connections
	meshShape.outMesh >> bakedGuidesNode.distributionMesh
	bakedGuidesNode.outputGuides >> guidesShape.inputGuides
	meshShape.outMesh >> guidesShape.distributionMesh

	return guidesShape

def AddEditGuidesToMesh( meshShape, guideCount = 10, rootGenerationMethod = 4, pointsPerStrandCount = 2, guideLength = 10.0, lengthRandomness = 0 ):
	transformNode = meshShape.firstParent2()
	guidesFromMesh = CreateNode( GuidesFromMeshNodeName )
	guidesShape = pm.createNode( HairShapeName, parent = transformNode )
	editGuidesShape = pm.createNode( EditGuidesShapeName, parent = transformNode )

	# Set parameters
	guidesFromMesh.attr( 'count' ).set( guideCount )
	guidesFromMesh.distribution.set( rootGenerationMethod )
	guidesFromMesh.pointCount.set( pointsPerStrandCount )
	guidesFromMesh.length.set( guideLength )
	guidesFromMesh.lengthRandomness.set( lengthRandomness )

	# Make connections
	meshShape.outMesh >> guidesFromMesh.inputMesh
	guidesFromMesh.outputGuides >> editGuidesShape.inputStrands
	guidesFromMesh.outputStack >> editGuidesShape.inputStack
	editGuidesShape.outputStrands >> guidesShape.inputHair
	meshShape.outMesh >> guidesShape.distributionMesh

	return editGuidesShape

def AddHairToNewPlane( strandCount = 10, rootGenerationMethod = 4, pointsPerStrandCount = 2, planeSegmentCount = 1, guideRootGenerationMethod = 4, guideCount = 10, planeSize = 1, length = 50, guideArea = 2 ):
	plane = pm.polyPlane( sx = planeSegmentCount, sy = planeSegmentCount, width = planeSize, height = planeSize )
	pm.select( plane )
	meshShape = plane[0]

	return AddHairToMesh( meshShape, strandCount, strandCount, rootGenerationMethod, pointsPerStrandCount, guideRootGenerationMethod, guideCount, 0, length, guideArea )

def AddHairToNewSphere( radius = 1, strandCount = 10, rootGenerationMethod = 4, pointsPerStrandCount = 2, sphereSegmentCount = 1, guideCount = 10, guideLength = 50, guideRootGenerationMethod = 4 ):
	sphere = pm.polySphere( radius = radius, sx = sphereSegmentCount, sy = sphereSegmentCount )
	pm.select( sphere )
	meshShape = sphere[0]

	return AddHairToMesh( meshShape, strandCount, strandCount, rootGenerationMethod, pointsPerStrandCount, guideCount = guideCount, length = guideLength, guideRootGenerationMethod = guideRootGenerationMethod )

def AddEditGuidesToNewPlane( planeSegmentCount = 1, guideCount = 10, rootGenerationMethod = 4, pointsPerStrandCount = 2, guideLength = 10.0, lengthRandomness = 0 ):
	plane = pm.polyPlane( sx = planeSegmentCount, sy = planeSegmentCount )
	pm.select( plane )
	meshShape = plane[0]

	return AddEditGuidesToMesh( meshShape, guideCount, rootGenerationMethod, pointsPerStrandCount, guideLength, lengthRandomness )

def AddEditGuidesToNewSphere( radius = 1, guideCount = 10, rootGenerationMethod = 4, pointsPerStrandCount = 2, segmentCount = 5, guideLength = 10.0 ):
	sphere = pm.polySphere( sx = segmentCount, sy = segmentCount, r = radius )
	pm.select( sphere )
	meshShape = sphere[0]

	return AddEditGuidesToMesh( meshShape, guideCount, rootGenerationMethod, pointsPerStrandCount, guideLength )

# Gets all vertices inside specified hair or guides in object coordinates
def GetVerticesInObjectCoordinates( strandsShape ):
	result = []
	for strandIndex in range( 0, pm.mel.OxGetStrandCount( strandsShape ) ):
		for pointIndex in range( 0, pm.mel.OxGetStrandPointCount( strandsShape, strandIndex ) ):
			result.append( pm.mel.OxGetStrandPointInObjectCoordinates( strandsShape, strandIndex, pointIndex ) )

	return result

# Gets all vertices inside specified hair or guides in world space coordinates
def GetVerticesInWorldSpace( strandsShape ):
	result = pm.mel.OxGetVertices( strandsShape, True )
	for strandIndex in range( 0, pm.mel.OxGetStrandCount( strandsShape ) ):
		for pointIndex in range( 0, pm.mel.OxGetStrandPointCount( strandsShape, strandIndex ) ):
			result.append( pm.mel.OxGetStrandPointInObjectCoordinates( strandsShape, strandIndex, pointIndex ) )

	return result

# Gets all vertices inside specified hair or guides in object coordinates and associates each strand with its unique strand id
def GetVerticesInObjectCoordinatesByStrandId( strandsShape ):
	result = {}
	strandIds = pm.mel.OxGetStrandIds( strandsShape )
	for strandIndex in range( 0, pm.mel.OxGetStrandCount( strandsShape ) ):
		verticesInObjectCoordinates = []
		for pointIndex in range( 0, pm.mel.OxGetStrandPointCount( strandsShape, strandIndex ) ):
			verticesInObjectCoordinates.append( pm.mel.OxGetStrandPointInObjectCoordinates( strandsShape, strandIndex, pointIndex ) )

		result[strandIds[strandIndex]] = verticesInObjectCoordinates

	return result

def GetStrandLengths( strandsShape ):
	result = []
	for strandIndex in range( 0, pm.mel.OxGetStrandCount( strandsShape ) ):
		strandLength = 0
		for pointIndex in range( 1, pm.mel.OxGetStrandPointCount( strandsShape, strandIndex ) ):
			strandLength += dt.Vector( pm.mel.OxGetStrandPointInObjectCoordinates( strandsShape, strandIndex, pointIndex ) ).distanceTo( pm.mel.OxGetStrandPointInObjectCoordinates( strandsShape, strandIndex, pointIndex - 1 ) )

		result.append( strandLength )

	return result

def GetStrandSegmentLengths( strandsShape ):
	result = []
	for strandIndex in range( 0, pm.mel.OxGetStrandCount( strandsShape ) ):
		for pointIndex in range( 1, pm.mel.OxGetStrandPointCount( strandsShape, strandIndex ) ):
			result.append( dt.Vector( pm.mel.OxGetStrandPointInObjectCoordinates( strandsShape, strandIndex, pointIndex ) ).distanceTo( pm.mel.OxGetStrandPointInObjectCoordinates( strandsShape, strandIndex, pointIndex - 1 ) ) )

	return result

def CheckPointsNearEqual( self, point1, point2, epsilon = 0.01, extraMessage = '' ):
	error = dt.Vector( point1 ).distanceTo( point2 )
	self.assertGreaterEqual( epsilon, error, msg = '[' + str( point1 ) + '] is not near equal [' + str( point2 ) + '], error: ' + str( error ) + extraMessage )

def CheckPointsNotNearEqual( self, point1, point2, epsilon = 0.01, extraMessage = '' ):
	error = dt.Vector( point1 ).distanceTo( point2 )
	self.assertLessEqual( epsilon, error, msg = '[' + str( point1 ) + '] is near equal [' + str( point2 ) + '], error: ' + str( error ) + extraMessage )

def CheckPointsAllNearEqual( self, points1, points2, epsilon = 0.01, points1Start = 0, points2Start = 0, pointsCount = -1, extraMessage = '' ):
	if pointsCount == -1:
		pointsCount = len( points1 ) - points1Start
		self.assertEqual( pointsCount, len( points2 ) - points2Start )

	for i in range( 0, pointsCount ):
		CheckPointsNearEqual( self, points1[i + points1Start], points2[i + points2Start], epsilon = epsilon, extraMessage = extraMessage )

# The vertices in object space should be equal and correctly assigned to the same strand ids
def CheckPointsAllNearEqualByStrandId( self, points1, points2, epsilon = 0.01, points1Start = 0, points2Start = 0, pointsCount = -1 ):
	for strandId, verticesInObjectSpace in points1.iteritems():
		point2 = points2[strandId]
		CheckPointsAllNearEqual( self, verticesInObjectSpace, point2, epsilon = epsilon, points1Start = points1Start, points2Start = points2Start, pointsCount = pointsCount, extraMessage = ' at strandId ' + str( strandId ) )

# Tests that each point within one specified set each has exactly one equal point in the other specified set
def CheckPointsMatch( self, points1, points2, epsilon = 0.01, pointsStart = 0, pointsCount = -1 ):
	self.assertEqual( len( points1 ), len( points2 ) )

	if pointsCount == -1:
		pointsCount = len( points1 ) - pointsStart

	wasMatched = [False] * pointsCount
	for i in range( 0, pointsCount ):
		hasMatch = False
		point = dt.Vector( points1[i + pointsStart] )
		for j in range( 0, pointsCount ):
			if( not wasMatched[j] and point.distanceTo( points2[j + pointsStart] ) <= epsilon ):
				hasMatch = True
				wasMatched[j] = True
				break

		self.assertTrue( hasMatch )

def CheckPointsAllNotNearEqual( self, points1, points2, epsilon = 0.01, pointsStart = 0, pointsCount = -1 ):
	self.assertEqual( len( points1 ), len( points2 ) )

	if pointsCount == -1:
		pointsCount = len( points1 ) - pointsStart

	for i in range( 0, pointsCount ):
		self.assertLessEqual( epsilon, dt.Vector( points1[i + pointsStart] ).distanceTo( points2[i + pointsStart] ), 'Points near equal at index {}, value {}'.format( i + pointsStart, points1[i + pointsStart] ) )

def CheckPointsNotAllNearEqual( self, points1, points2, epsilon = 0.01, pointsStart = 0, pointsCount = -1 ):
	self.assertEqual( len( points1 ), len( points2 ) )
	allPointsAreEqual = True

	if pointsCount == -1:
		pointsCount = len( points1 ) - pointsStart

	for i in range( 0, pointsCount ):
		if( dt.Vector( points1[pointsStart + i] ).distanceTo( points2[pointsStart + i] ) > epsilon ):
			allPointsAreEqual = False
			break
	self.assertFalse( allPointsAreEqual, 'All points are equal, expecting at least one not to be equal' )

def CheckUniqueStrandIds( self, hairShape ):
	strandIds = pm.mel.OxGetStrandIds( hairShape )
	idSet = set()
	for id in strandIds:
		if id in idSet:
			self.fail( "Duplicate strand ID {}".format( id ) )
		idSet.add( id )

def GetVertexNormals (meshShape):
	selection = pm.select( meshShape )
	vtxCount = pm.polyEvaluate (v= True)
	normals = []
	for vtx in range(vtxCount):
		pm.select (meshShape+".vtx[" +str(vtx) +"]")
		normal = pm.polyNormalPerVertex( query = True, xyz = True)
		normals.append(normal[0:3])

	return normals

def GetAveragePointDistance( self, points1, points2 ):
	self.assertEqual( len( points1 ), len( points2 ) )
	result = 0.0
	for i in range( 0, len( points1 ) ):
		result += dt.Vector( points1[i] ).distanceTo( points2[i] )

	result /= len( points1 )
	return result

def GetAveragePointMove( self, points1, points2 ):
	self.assertEqual( len( points1 ), len( points2 ) )
	averageMove = dt.Vector( 0, 0, 0)
	for i in range( len( points1 ) ):
		averageMove += dt.Vector( points2[i] ) - dt.Vector( points1[i] )
	averageMove /= len( points1 )
	return averageMove

def GetMaxPointDistance( self, points1, points2 ):
	self.assertEqual( len( points1 ), len( points2 ) )
	result = 0.0
	for i in range( 0, len( points1 ) ):
		distance = dt.Vector( points1[i] ).distanceTo( points2[i] )
		if distance > result:
			result = distance
	return result

def GetBoundingBox( points ):
	smallestPoint = [99999, 99999, 99999]
	biggestPoint = [-99999, -99999, -99999]

	for i in range( 0, len( points ) ):
		for j in range( 0, 3 ):
			if points[i][j] < smallestPoint[j]:
				smallestPoint[j] = points[i][j]
			if points[i][j] > biggestPoint[j]:
				biggestPoint[j] = points[i][j]

	return [smallestPoint, biggestPoint]

def GetMeshVertices( mesh ):
	result = []
	index = 0

	for vertexIndex in range( 0, len( mesh.vtx ) ):
		result.append( mesh.vtx[vertexIndex].getPosition() )
		vertexIndex += 1

	return result

def GetTipPositions( strandsShape, useObjectCoordinates = False ):
	result = []
	for strandIndex in range( 0, pm.mel.OxGetStrandCount( strandsShape ) ):
		if( useObjectCoordinates ):
			result.append( pm.mel.OxGetStrandPointInObjectCoordinates( strandsShape, strandIndex, pm.mel.OxGetStrandPointCount( strandsShape, strandIndex ) - 1 ) )
		else:
			result.append( pm.mel.OxGetStrandPoint( strandsShape, strandIndex, pm.mel.OxGetStrandPointCount( strandsShape, strandIndex ) - 1 ) )

	return result

def GetRootPositions( strandsShape, useObjectCoordinates = True ):
	result = []
	for strandIndex in range( 0, pm.mel.OxGetStrandCount( strandsShape ) ):
		if( useObjectCoordinates ):
			result.append( pm.mel.OxGetStrandPointInObjectCoordinates( strandsShape, strandIndex, 0 ) )
		else:
			result.append( pm.mel.OxGetStrandPoint( strandsShape, strandIndex, 0 ) )

	return result

def GetStrandTangents( strandsShape, useObjectCoordinates = False ):
	result = []
	roots = GetRootPositions( strandsShape, useObjectCoordinates )
	tips = GetTipPositions( strandsShape, useObjectCoordinates )

	for strandIndex in range( 0, len( roots ) ):
		result.append( ( dt.Vector( tips[strandIndex] ) - dt.Vector( roots[strandIndex] ) ).normal() )

	return result

def GetStrandPointPositionsAtIndex( strandsShape, pointIndex, useObjectCoordinates = False ):
	result = []
	for strandIndex in range( 0, pm.mel.OxGetStrandCount( strandsShape ) ):
		if( useObjectCoordinates ):
			result.append( pm.mel.OxGetStrandPointInObjectCoordinates( strandsShape, strandIndex, pointIndex ) )
		else:
			result.append( pm.mel.OxGetStrandPoint( strandsShape, strandIndex, pointIndex ) )

	return result

def GetStrandPoints( strandsShape, strandIndex, useObjectCoordinates = False ):
	pointGetter = ( lambda pointIndex: pm.mel.OxGetStrandPointInObjectCoordinates( strandsShape, strandIndex, pointIndex ) ) if useObjectCoordinates else \
		( lambda pointIndex: pm.mel.OxGetStrandPoint( strandsShape, strandIndex, pointIndex ) )
	return map( pointGetter, range( 0, pm.mel.OxGetStrandPointCount( strandsShape, strandIndex ) ) )

def TestStrandOperatorChangesGuideCount( self, nodeName, pointsPerStrandCount = 2, isTestingOnGuides = True, strandCount = 10, rootGenerationMethod = 4, operatorSetup = None ):
	guidesShape = AddGuidesToNewPlane( pointsPerStrandCount = pointsPerStrandCount ) if isTestingOnGuides else \
		AddHairToNewPlane( pointsPerStrandCount = pointsPerStrandCount, strandCount = strandCount, rootGenerationMethod = rootGenerationMethod )

	verticesBefore = GetVerticesInObjectCoordinates( guidesShape )
	verticesBeforeCount = len( verticesBefore )
	self.assertGreaterEqual( verticesBeforeCount, 2, 'Expected at least two vertices, instead got: %(vertexCount)s' % { 'vertexCount': verticesBeforeCount } )

	operatorNodeName = pm.mel.OxAddStrandOperator( guidesShape, nodeName )

	if operatorSetup != None:
		operatorSetup( operatorNodeName )

	verticesAfter = GetVerticesInObjectCoordinates( guidesShape )

	self.assertNotEqual( len( verticesAfter ), len( verticesBefore ), 'Vertex count was not modified by the operator' )

def TestStrandOperatorChangingGuides( self, nodeName, isCheckingAllVertices = True, epsilon = 0.1, pointsPerStrandCount = 2, isTestingOnGuides = True, strandCount = 10, rootGenerationMethod = 4, operatorSetup = None ):
	guidesShape = AddGuidesToNewPlane( pointsPerStrandCount = pointsPerStrandCount ) if isTestingOnGuides else \
		AddHairToNewPlane( pointsPerStrandCount = pointsPerStrandCount, strandCount = strandCount, rootGenerationMethod = rootGenerationMethod )

	verticesBefore = GetVerticesInObjectCoordinates( guidesShape )
	verticesBeforeCount = len( verticesBefore )
	self.assertGreaterEqual( verticesBeforeCount, 2, 'Expected at least two vertices, instead got: %(vertexCount)s' % { 'vertexCount': verticesBeforeCount } )

	operatorNodeName = pm.mel.OxAddStrandOperator( guidesShape, nodeName )

	if operatorSetup != None:
		operatorSetup( operatorNodeName )

	verticesAfter = GetVerticesInObjectCoordinates( guidesShape )

	self.assertEqual( len( verticesAfter ), len( verticesBefore ), 'Vertex count was modified by the operator' )

	wasAtLeastOneVertexDifferent = False
	for vertexIndex in range( 0, len( verticesBefore ) ):
		distance = dt.Vector( verticesAfter[vertexIndex] ).distanceTo( verticesBefore[vertexIndex] )
		if isCheckingAllVertices:
			self.assertGreater( distance, epsilon,
				'vertexIndex: %(vertexIndex)d, vertexPosition: %(tipPosition)s, expected anything but: %(expected)s' % { 'vertexIndex': vertexIndex, 'tipPosition': verticesAfter[vertexIndex], 'expected': verticesBefore[vertexIndex] } )
		elif not wasAtLeastOneVertexDifferent:
			wasAtLeastOneVertexDifferent = distance > epsilon

	if not isCheckingAllVertices:
		self.assertTrue( wasAtLeastOneVertexDifferent, 'Operator did not change any vertices' )

class RenderModeScope:
	"""
	A "context manager" to be used with Python's 'with' statement. See https://docs.python.org/2/reference/datamodel.html#context-managers
	Wraps the code within the 'with' statement in OxSetIsRendering(true) / OxSetIsRendering(false) calls
	Typical usage:
		\code
		with TestUtilities.RenderModeScope():
			# some code that will run with Ornatrix set in render mode
		\endcode
	"""
	def __enter__( self ):
		pm.mel.OxSetIsRendering(True)

	def __exit__( self, type, value, traceback ):
		pm.mel.OxSetIsRendering(False)


def CreateSingleHairWithPropagator( count = 4, length = 2, sideCount = 2 ):
	resultShape = AddGuidesToNewPlane( guideCount = 1, pointsPerStrandCount = 3, rootGenerationMethod = 2, planeSegmentCount = 1, length = 5 )
	propagatorNode = pm.PyNode( pm.mel.OxAddStrandOperator( resultShape, PropagationNodeName ) )
	propagatorNode.setAttr( 'count', count )
	propagatorNode.verticesPerRootCount.set( 4 )
	propagatorNode.textureMappingMethod.set( 2 )
	propagatorNode.generationMethod.set( 0 )
	propagatorNode.length.set( length )
	propagatorNode.lengthRandomness.set( 0 )
	propagatorNode.sideCount.set( sideCount )
	propagatorNode.setAttr( "lengthRamp[0].lengthRamp_FloatValue", 1 )
	propagatorNode.setAttr( "lengthRamp[1].lengthRamp_FloatValue", 1 )
	propagatorNode.setAttr( "lengthRamp[2].lengthRamp_FloatValue", 1 )
	propagatorNode.setAttr( "distributionRamp[0].distributionRamp_FloatValue", 1 )
	propagatorNode.setAttr( "distributionRamp[1].distributionRamp_FloatValue", 1 )
	propagatorNode.setAttr( "distributionRamp[2].distributionRamp_FloatValue", 1 )
	return resultShape

def GetTextureCoordinates( strandsShape, channel, type ):
	result = []
	for strandIndex in range( 0, pm.mel.OxGetStrandCount( strandsShape ) ):
		for pointIndex in range( 0, pm.mel.OxGetStrandPointCount( strandsShape, strandIndex ) ):
			result.append( pm.mel.OxGetTextureCoordinate( strandsShape, strandIndex, pointIndex, channel, type = type ) )
	return result

def GetTextureCoordinatesByStrandId( self, strandsShape, channel, type ):
	result = {}
	strandIds = pm.mel.OxGetStrandIds( strandsShape )
	for strandIndex in range( 0, pm.mel.OxGetStrandCount( strandsShape ) ):
		strandTextureCoordinates = []
		for pointIndex in range( 0, pm.mel.OxGetStrandPointCount( strandsShape, strandIndex ) ):
			strandTextureCoordinates.append( pm.mel.OxGetTextureCoordinate( strandsShape, strandIndex, pointIndex, channel, type = type ) )

		result[strandIds[strandIndex]] = strandTextureCoordinates

	return result

def GetReferencePropagatorTextureCoordinates( stemVertexCount, branchVertexCount, branchesPerStemCount, sideCount = 2, islandCount = 1, repetitions = 1 ):
	"""Obtains reference coordinates for a texture atlas with a uniform flat-map propagator."""
	stemVerticesV = [ float( index ) / ( stemVertexCount - 1 ) for index in range( stemVertexCount ) ]
	branchVerticesU = [ float( index ) / ( branchVertexCount - 1 ) / islandCount / sideCount for index in range( branchVertexCount ) ]
	result = []
	stemOffsetU = 1.0 / ( sideCount * islandCount )
	for islandIndex in range( islandCount ):
		islandResult = []
		uOffset = float( islandIndex ) / islandCount
		for vertexV in stemVerticesV:
			islandResult.append( dt.Vector( stemOffsetU + uOffset, vertexV, 0.0 ) )
		for branchIndex in range( branchesPerStemCount ):
			vOffset = float( branchIndex ) / ( branchesPerStemCount - 1 )
			sideIndex = branchIndex % sideCount
			uOffsetBranch = stemOffsetU * ( ( sideIndex / 2 ) * 2 + 1 ) + uOffset
			direction = -1 if sideIndex % 2 == 0 else 1
			for vertexU in branchVerticesU:
				islandResult.append( dt.Vector( uOffsetBranch + vertexU * direction, vOffset, 0.0 ) )
		for _ in range( repetitions ):
			result.extend( islandResult )
	return result

