import json
from HairSystemNode import *

class HairSystem(object):

	def __init__(self, name, path):
		self.jsonData = None
		self.name = name
		self.path = path
		self.nodes = {}

	def addNode(self, node):
		self.nodes[node.uid] = node



	def insertHostNode(self, hostNode):
		hostNode.container.bridge.insertNodeInHairSystem(self, hostNode)
		newNode = HairSystemNode(hostNode.name)
		newNode.uid = hostNode.uid
		newNode.type = "hostNode"
		newNode._isHostNode = True
		newNode.setHostNode(hostNode)
		self.addNode(newNode)
		return newNode

	def unbindHostNode(self):
		for node in self.nodes.itervalues():
			if node.isHostNode():
				node.setHostNode(None)

	def bindHostNode(self, hostNode):
		if hostNode.uid in self.nodes:
			self.nodes[hostNode.uid].setHostNode(hostNode)
		else:
			self.insertHostNode(hostNode)

	def clean(self):
		toRemove = []
		for key, node in self.nodes.iteritems():
			if node.isHostNode() and (not node.getHostNode() or node.getHostNode()._deleted):
				print "removing"
				toRemove.append(key)
		for rem in toRemove:
			del self.nodes[rem]

	def getNodes(self):
		return self.nodes.itervalues()

	def serialize(self):
		data = {"nodes": {}}
		for node in self.getNodes():
			nodeInfo = {}
			nodeInfo['name'] = node.name
			nodeInfo['type'] = node.type
			nodeInfo['data'] = node.getDatas()
			data['nodes'][node.uid] = nodeInfo
		return json.dumps(data)

	def deserialize(self, rawData):
		if rawData != None:
			jsonData = json.loads(rawData)
			if "nodes" in jsonData:
				for uid, node in jsonData['nodes'].iteritems():
					newNode = HairSystemNode(node['name'])
					newNode.uid = uid
					newNode.type = node['type']
					newNode.setDatas(node['data'])
					self.nodes[uid] = newNode
