class nodeLayoutContainer(object):
	def __init__(self, autoLayout, node):
		self.level = 0
		self.processed = False
		self.autoLayout = autoLayout
		self.node = node

	def isTerminal(self):
		return self.node.pinManager.hasConnectedOutput() == False

	def getInputedNode(self):
		inputedNode = self.node.pinManager.getInputedNode()
		result = []
		for node in inputedNode:
			if node in self.autoLayout.layoutObject:
				result.append(self.autoLayout.layoutObject[node])
		return result


class AutoLayout(object):
	def __init__(self, nodes):
		self.layoutObject = {}
		self.baseSeparation = 120
		for node in nodes:
			self.layoutObject[node] = nodeLayoutContainer(self, node)

		self.grid = {}

	def layoutRecursive(self, node):
		if node.processed:
			return
		node.processed = True
		inputedNode = node.getInputedNode()
		for inNode in inputedNode:
			inNode.level = node.level + 1
		for inNode in inputedNode:
			self.layoutRecursive(inNode)

	def doLayout(self):
		terminalNode = self.getTerminalNode()
		for node in terminalNode:
			self.layoutRecursive(node)

		for n in self.layoutObject.values():
			if n.level not in self.grid:
				self.grid[n.level] = []
			self.grid[n.level].append(n)
			n.node.setPos(len(self.grid[n.level]) * self.baseSeparation, n.level * self.baseSeparation)

	def getTerminalNode(self):
		terminalNode = []
		for key in self.layoutObject:
			node = self.layoutObject[key]
			if node.isTerminal():
				terminalNode.append(node)
		return terminalNode
