from Ornatrix.QtImport import *
from Breadcrumb import *
from NodeCreationGrid import NodeCreationGrid

class GraphMenu(QtWidgets.QWidget):

	def __init__(self, container, parent=None):
		super(GraphMenu, self).__init__()
		self.container = container
		self.container.registerUI("graphMenu", self)

		self.setSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)

		self.buildUI()

	def buildUI(self):
		mainLayout = QtWidgets.QVBoxLayout()

		hbox = QtWidgets.QHBoxLayout()
		creationGrid = NodeCreationGrid(self.container)
		hbox.addWidget(creationGrid)


		mainLayout.addLayout(hbox)

		hbox = QtWidgets.QHBoxLayout()
		self.hairSelector = QtWidgets.QComboBox()
		self.hairSelector.currentIndexChanged.connect(self.hairSelected)

		hbox.addWidget(self.hairSelector)

		createNewGraph = QtWidgets.QPushButton("Create New Graph Network")
		createNewGraph.clicked.connect(self.createGraphDialog)
		hbox.addWidget(createNewGraph)


		mainLayout.addLayout(hbox)

		breadcrumb = Breadcrumb(self.container)
		mainLayout.addWidget(breadcrumb)

		self.setLayout(mainLayout)

	def hairSelected(self):
		self.container.currenthairSystem = self.hairSelector.currentIndex()
		self.container.update()

	def updateUI(self):
		self.hairSelector.clear()
		hairSystems = self.container.bridge.getHairSystem()
		for hs in hairSystems:
			self.hairSelector.addItem(hs.path)

	def toggleGraph(self):
		open = self.graphButton.isChecked()
		self.container.editor.displayGraph(open)

	def update(self, force):
		pass


	def createGraphDialog(self):
		dialog = QtWidgets.QDialog()
		def accept(click):
			dialog.accept()
			self.createGraph(click.property('choice'))
		mode = QtWidgets.QDialogButtonBox()
		allButton = QtWidgets.QPushButton("Insert All Stack")
		allButton.setProperty('choice', 0)
		selectButton = QtWidgets.QPushButton("Insert Selected Stack")
		selectButton.setProperty('choice', 1)
		emptyButton = QtWidgets.QPushButton("Empty HairSystem")
		emptyButton.setProperty('choice', 2)
		mode.addButton(allButton, QtWidgets.QDialogButtonBox.AcceptRole)
		mode.addButton(selectButton, QtWidgets.QDialogButtonBox.AcceptRole)
		mode.addButton(emptyButton, QtWidgets.QDialogButtonBox.AcceptRole)
		mode.clicked.connect(accept)
		layout = QtWidgets.QVBoxLayout()
		layout.addWidget(mode)
		dialog.setLayout(layout)
		dialog.exec_()

	def createGraph(self, mode):
		hairSystem = self.container.bridge.createHairSystem()
		if mode == 0:
			nodes = self.container.bridge.getAllOxNode()
			for node in nodes:
				self.container.bridge.insertNodeInHairSystem(hairSystem, node)
		if mode == 1:
			nodes = self.container.bridge.getNetwork([self.container.bridge.getSelectedOxNodes()[0]])
			for node in nodes:
				self.container.bridge.insertNodeInHairSystem(hairSystem, node)
		self.container.bridge.dataChanged()


