from .Connector import Connector
from .DummiePin import DummiePin


class ConnectionBuilder(object):

	def __init__(self, graph):
		self.graph = graph
		self.asSource = False
		self.basePin = None
		self.initialConnection = None
		self.confirmDisconnect= None

	def prepareConnection(self, pin):
		self.graph.grabStarted.emit(pin)
		self.basePin = pin
		if self.basePin.isOutput():
			self.asSource = True
		else:
			self.asSource = False
			for connection in self.basePin.connection:
				self.confirmDisconnect = connection
				connection.doDisconnect()

		self.createFakeConnection(self.basePin)

	def grabFinished(self):
		self.graph.container.bridge.startUndo()
		self.graph.grabFinished.emit()
		self.deleteFakeConnection()
		if self.connectionGrabber.hoverInputPin is not None:
			if self.asSource:
				DestPin = self.connectionGrabber.hoverInputPin()
				SrcPin = self.basePin
				for connection in DestPin.connection:
					self.doDisconnect(connection)
			else:
				DestPin = self.basePin
				SrcPin = self.connectionGrabber.hoverInputPin()

			if DestPin is not None and SrcPin is not None:
				self.doConnect(SrcPin, DestPin)
		# If not connection was found, we display the node creation menu
		else:
			posOri = self.connectionGrabber.pos()
			pos = self.graph.views()[0].mapFromScene(self.connectionGrabber.pos())
			pos = self.graph.views()[0].viewport().mapToGlobal(pos)
			if self.connectionGrabber.doMenu:
				from ..Editor.Graph.NodeCreationMenu import NodeCreationMenu
				menu = NodeCreationMenu(self, pos)
				result = menu.exec_(pos)
				if result is not None:
					node = self.graph.createNode(Bridge.createNode(result.data(), posOri.x(), posOri.y()),
					                             result.data())
					destPin = node.pinManager._inputPins.itervalues().next()
					self.doConnect(self.basePin, destPin)
		self.cleanUp()
		self.graph.container.bridge.endUndo()

	def createFakeConnection(self, pin):
		self.connectionGrabber = DummiePin(pin, self)
		self.graph.addItem(self.connectionGrabber)
		self.connector = Connector(self.basePin, self.connectionGrabber, self.graph.container)
		self.connector.doConnect(False)
		self.connectionGrabber.grabMouse()
		self.graph.addItem(self.connector)

	def deleteFakeConnection(self):
		self.connector.doDisconnect(False)
		self.graph.removeItem(self.connector)

	def doConnect(self, outputPin, inputPin):
		if self.confirmDisconnect:
			self.doDisconnect(self.confirmDisconnect)
			self.confirmDisconnect = None
		connection = Connector(outputPin, inputPin, self.graph.container)
		connection.doConnect()
		self.graph.addItem(connection)
		self.graph.container.bridge.connectNode(connection.outputPin().node.nodeContainer.getHostNode(),
		                                        connection.inputPin().node.nodeContainer.getHostNode(),
		                                        connection.inputPin().id,
		                                        connection.outputPin().id)

	def doDisconnect(self, connection):
		connection.doDisconnect()
		self.graph.removeItem(connection)
		self.graph.container.bridge.disconnectNode(connection.outputPin().node.nodeContainer.getHostNode(),
		                                           connection.inputPin().node.nodeContainer.getHostNode(),
		                                           connection.inputPin().id,
		                                           connection.outputPin().id)

	def cleanUp(self):
		if self.confirmDisconnect:
			self.doDisconnect(self.confirmDisconnect)
			self.confirmDisconnect = None
		self.graph.removeItem(self.connectionGrabber)
		self.graph.removeItem(self.connector)
		self.connectionGrabber = None
		self.connector = None
