from Ornatrix.QtImport import *

from ..ConnectionContextMenu import *

import weakref


class Connector(QtWidgets.QGraphicsPathItem):
	defaultColor = QtGui.QColor(255, 134, 3)
	__defaultPen = QtGui.QPen(defaultColor, 1.5)
	colorOverride = None

	def __init__(self, outputPin, inputPin, container):
		super(Connector, self).__init__()

		penStyle = QtCore.Qt.DashLine

		self.inputPin = weakref.ref(inputPin)
		self.outputPin = weakref.ref(outputPin)
		self.container = container

		self.setAcceptHoverEvents(True)

		self.isConnected = False

		self.setPen(self.__defaultPen)
		self.setZValue(-1)

	def getPath(self):
		srcPoint = self.mapFromScene(self.getSourcePin().scenePos())
		srcPoint2 = QtCore.QPointF(srcPoint)
		srcPoint2.setY(srcPoint2.y() - 50)

		dstPoint = self.mapFromScene(self.getDestPin().scenePos())
		dstPoint2 = QtCore.QPointF(dstPoint)
		dstPoint2.setY(dstPoint.y() + 50)

		dist_between = dstPoint - srcPoint

		path = QtGui.QPainterPath()
		path.moveTo(srcPoint)
		path.cubicTo(srcPoint2, dstPoint2, dstPoint)

		return path

	def shape(self):
		path = self.getPath()
		stroker = QtGui.QPainterPathStroker()
		stroker.setWidth(10)
		stroked = stroker.createStroke(path)
		return stroked.united(path)

	def paint(self, painter, option, widget):
		painter.setRenderHint(QtGui.QPainter.Antialiasing)
		painter.setRenderHint(QtGui.QPainter.SmoothPixmapTransform)
		if self.isConnected:
			self.prepareGeometryChange()

			'''if self.isSelected():
				self.setPen(QPen(Qt.black, 4))'''
			self.setPath(self.getPath())
			if self.colorOverride:
				pen = self.pen()
				pen.setColor(self.colorOverride)
				self.setPen(pen)

			self.inputPin().update()
			self.outputPin().update()

			QtWidgets.QGraphicsPathItem.paint(self, painter, option, widget)

	# Build Node creation menu
	def contextMenuEvent(self, event):
		menu = ConnectionContextMenu(self, event)

	def hoverEnterEvent(self, event):
		pen = self.pen()
		pen.setWidth(4)
		self.setPen(pen)
		super(Connector, self).hoverEnterEvent(event)

	def hoverLeaveEvent(self, event):
		self.setPen(self.__defaultPen)
		super(Connector, self).hoverLeaveEvent(event)

	def doDisconnect(self, emitSignel=True):
		self.isConnected = False
		if self.inputPin():
			self.inputPin().disconnectConnector(self, emitSignel)
		if self.outputPin():
			self.outputPin().disconnectConnector(self, emitSignel)

	def doConnect(self, emitSignel=True):
		self.isConnected = True
		if self.inputPin():
			self.inputPin().connectConnector(self, emitSignel)
		if self.outputPin():
			self.outputPin().connectConnector(self, emitSignel)

	def getSourceNode(self):
		return self.getSourcePin().getNode()

	def getDestNode(self):
		return self.getDestPin().getNode()

	def getSourcePin(self):
		if self.inputPin().isOutput():
			return self.inputPin()
		return self.outputPin()

	def getDestPin(self):
		if self.inputPin().isInput():
			return self.inputPin()
		return self.outputPin()
