from .Pin import *

import weakref


class DummiePin(QtWidgets.QGraphicsItem):

	def __init__(self, otherPin, connectionHandler):
		super(DummiePin, self).__init__()

		self.otherPin = otherPin
		self.connectionHandler = connectionHandler
		self.hoverInputPin = None

		self.setPos(otherPin.scenePos())
		self.doMenu = False

		self.mouseReleased = False

		self.hasMoved = False

		self.w = 20
		self.h = 20

	def isOutput(self):
		return self.otherPin.isInput()

	def isInput(self):
		return self.otherPin.isOutput()

	def boundingRect(self):
		rect = QtCore.QRectF(-5, -5, 10, 10)
		rect.adjusted(1, 1, 1, 1)
		return rect

	def mouseMoveEvent(self, event):
		self.hasMoved = True
		if not self.mouseReleased:
			self.setPos(self.mapToScene(event.pos()))

			collidingItems = self.collidingItems(QtCore.Qt.IntersectsItemBoundingRect)

			if len(collidingItems) > 0 and issubclass(collidingItems[0].__class__, Pin):
				if (self.checkTypeValidity(collidingItems[0])):
					self.hoverInputPin = weakref.ref(collidingItems[0])
				else:
					self.hoverInputPin = None
			else:
				self.hoverInputPin = None

	def mousePressEvent(self, event):
		if event.button() == Qt.RightButton:
			event.accept()
			self.doMenu = True
			self.ungrabMouse()
			self.connectionHandler.grabFinished()

	def checkTypeValidity(self, hoverPin):
		if hoverPin == self.otherPin:
			return False
		if self.otherPin.parentItem() == hoverPin.parentItem():
			return False
		if self.otherPin.isOutput():
			if hoverPin.isOutput():
				return False
			sourcePin = self.otherPin
			destPin = hoverPin
			return True
		else:
			if hoverPin.isInput():
				return False
			destPin = self.otherPin
			sourcePin = hoverPin
			# if destPin.acceptedType in sourcePin.types:
			return True
		return False

	def mouseReleaseEvent(self, event):
		self.ungrabMouse()
		self.connectionHandler.grabFinished()

	# Handle Parenting operation when connected
	def connectConnector(self, connector, emit=True):
		pass

	def disconnectConnector(self, connector, emit=True):
		pass

	def paint(self, painter, option, widget):
		brush = QtGui.QBrush(QtCore.Qt.white)
		painter.setBrush(brush)
		painter.setPen(QtGui.QPen(QtCore.Qt.black, 1, QtCore.Qt.SolidLine, QtCore.Qt.RoundCap, QtCore.Qt.RoundJoin))
		painter.drawEllipse(QtCore.QPointF(0, 0), 5, 5)
