from .Node import *

class HostNode(Node):
	nodeColor = QtGui.QColor(77, 77, 77)
	defaultPainter = PinPainter()

	def __init__(self, node):
		super(HostNode, self).__init__(node)
		self._pinManager = PinManager(self)

		self.configure()

	def configure(self):
		i = 0
		for input in self.nodeContainer.getHostNode().getSupportedInput():
			inputPin = Pin(i, input[0], True, self)
			inputPin.setAcceptedType(input[1])
			self.addPin(inputPin)
			i += 1

		i = 0
		for output in self.nodeContainer.getHostNode().getSupportedOutput():
			outputPin = Pin(i, output[0], False, self)
			outputPin.setType(output[1])
			self.addPin(outputPin)
			i += 0

		if self.nodeContainer.getHostNode().supportState():
			self.hideButton = Button(self)
			self.hideButton.setPos(-self.width / 2, 0)
			self.hideButton.onClicked.connect(self.toggleEnable)

	def addPin(self, pin):
		if pin.isInput():
			pin.onConnected.connect(self.onPinConnected)
			pin.onDisconnected.connect(self.onPinDisconnected)
		self.pinManager.addPin(pin)
		return pin

	def onPinConnected(self, connector):
		pass

	def onPinDisconnected(self, connector):
		pass

	def toggleEnable(self):
		self.enabled = self.nodeContainer.getHostNode().toggleNode()

	def isEnabled(self):
		return self.nodeContainer.getHostNode().enabled

	###
	### Drawing
	###
	def paint(self, painter, option, widget):
		painter.setRenderHint(QtGui.QPainter.Antialiasing)
		painter.setRenderHint(QtGui.QPainter.SmoothPixmapTransform)

		brush = QtGui.QBrush(self.nodeColor, QtCore.Qt.SolidPattern)

		if self.isSelected():
			self.nodePen.setColor(QtGui.QColor(0, 255, 255))
		elif self.borderOverrided:
			self.nodePen.setColor(self.penColorOverride)
		else:
			self.nodePen.setColor(QtGui.QColor(100, 100, 100))

		painter.setBrush(brush)
		painter.setPen(self.nodePen)

		path = QtGui.QPainterPath()
		path.addRoundedRect(QtCore.QRectF(-self.width / 2, -self.height / 2, self.width, self.height), 15, 15)


		painter.fillPath(path, self.nodeColor)
		painter.drawPath(path)

		if self.icon is not None:
			painter.drawPixmap(QtCore.QRectF(-20, -20, 40, 40), self.icon,
			                   QtCore.QRectF(0, 0, self.icon.size().width(), self.icon.size().height()))

		# NodeName
		brush = QtGui.QBrush(QtGui.QColor(0, 0, 0), QtCore.Qt.SolidPattern)
		brush.setColor(QtGui.QColor(0, 0, 0))
		painter.setBrush(brush)
		painter.setPen(QtGui.QPen(QtCore.Qt.white, 1, QtCore.Qt.SolidLine, QtCore.Qt.RoundCap, QtCore.Qt.RoundJoin))
		painter.drawText(QtCore.QRectF(self.width / 2 + 10, -10, 140, 20), self.name)

		#Time
		totalTime = self.scene().totalTime + 0.0001
		mult = float(self.nodeContainer.getHostNode().computeTime) / totalTime
		painter.setPen(QtGui.QPen(QtGui.QColor(mult*255, (1 - mult)*255, 0), 1, QtCore.Qt.SolidLine, QtCore.Qt.RoundCap, QtCore.Qt.RoundJoin))
		painter.drawText(QtCore.QRectF(self.width / 2 + 10, 10, 140, 20), str(float(self.nodeContainer.getHostNode().computeTime) / 1000) + " s")


		self.textBB = painter.boundingRect(QtCore.QRectF(self.width / 2 + 10, -10, 140, 20), 0, self.name)

	###
	### Getter/Setter
	###
	@property
	def pinManager(self):
		return self._pinManager

	def getConnector(self):
		return self._pinManager.getAllConnector()