from Ornatrix.QtImport import *

from .PinManager import *
from .PinPainter import *

from .Pin import *
from ..NodeContextMenu import NodeContextMenu

from .Button import *


class Node(QtWidgets.QGraphicsItem):
	nodeColor = QtGui.QColor(20, 20, 20)
	nodePen = QtGui.QPen(QtCore.Qt.gray, 2, QtCore.Qt.SolidLine, QtCore.Qt.RoundCap, QtCore.Qt.RoundJoin)

	height = 60
	width = 60

	"""\brief base class which implements dragging and scaling of content and line connection to parent  """

	def __init__(self, nodeContainer):
		super(Node, self).__init__()

		self.__editor = None


		# self.createOutputPin('output1')

		self.icon = nodeContainer.icon()

		self.nodeContainer = nodeContainer
		self.uid = nodeContainer.uid
		self.name = nodeContainer.name


		self.size = QtCore.QSize(self.height, self.width)
		self.borderOverrided = False
		self.connectionColor = QtGui.QColor(100, 0, 0)
		self.enabled = True
		self.textBB = QtCore.QRect(0, 0, 0, 0)

		# in screen coords
		self.setPos(self.nodeContainer.getPos()["x"], self.nodeContainer.getPos()["y"])
		self.setFlag(QtWidgets.QGraphicsItem.ItemIsSelectable, True)
		self.setFlag(QtWidgets.QGraphicsItem.ItemIsMovable)
		self.setFlag(QtWidgets.QGraphicsItem.ItemSendsGeometryChanges, True)

		self.setAcceptHoverEvents(True)

		self.size = QtCore.QSize(100, 100)

		self.moveMode = False
		self.ignore = False

	###
	### Geometry
	###

	def dragRect(self):
		return QtCore.QRectF(QtCore.QRect(-self.width / 2, -self.height / 2, self.width, self.height))

	def boundingRect(self):
		boundingRect = QtCore.QRectF(QtCore.QRect(-self.width / 2, -self.height / 2, self.width + 10, self.height))
		boundingRect = boundingRect.united(self.childrenBoundingRect())
		boundingRect = boundingRect.united(self.textBB)
		boundingRect = boundingRect.adjusted(-2, -2, 2, 2)
		return boundingRect

	###
	### Event
	###

	def contextMenuEvent(self, event):
		menu = NodeContextMenu(self, event.screenPos())

	def mousePressEvent(self, event):
		# do not react on events on arrows

		self.scene().nodeSelected(self)

		dragRect = self.mapToScene(self.dragRect())
		self.hasMoved = False
		if event.button() == QtCore.Qt.LeftButton:
			if dragRect.containsPoint(QtCore.QPointF(event.scenePos().x(), event.scenePos().y()),
			                          QtCore.Qt.WindingFill):
				self.moveMode = True
				self.lastPos = event.scenePos()
		else:
			event.ignore()

	def mouseReleaseEvent(self, event):
		self.moveMode = False

	def mouseMoveEvent(self, event):
		if self.ignore:
			event.ignore()

		self.hasMoved = True

		if self.moveMode:
			self.prepareGeometryChange()
			p = event.scenePos() - self.lastPos
			self.lastPos = event.scenePos()
			self.scene().nodeMoving(self, p)
			# self.setPos(p)
			# self.update()

	def itemChange(self, change, val):
		if change == QtWidgets.QGraphicsItem.GraphicsItemChange.ItemPositionChange:
			self.nodeContainer.setPos(val.x(), val.y())
			return val
		return super(Node, self).itemChange(change, val)

	def updateNode(self):
		pass

	@property
	def editor(self):
		return self.__editor

	@editor.setter
	def editor(self, editor):
		self.__editor = editor
