from Ornatrix.QtImport import *

from PinPainter import *


class Pin(QtWidgets.QGraphicsObject):
	onConnected = QtCore.Signal(object)
	onDisconnected = QtCore.Signal(object)

	baseColor = QtCore.Qt.darkRed
	connectedColor = QtCore.Qt.red
	overrideColor = None

	# defaultPainter = PinPainter()

	def __init__(self, id, name, input, parentNode):
		super(Pin, self).__init__(parent=parentNode)

		self.id = id
		self.name = name
		self.node = parentNode
		self.connection = []
		self._isInput = input
		self._type = None
		self._acceptedTypes = []

		self.shapeFunc = None
		self.pathShape = None

		# in parent coords
		self.setPos(QtCore.QPoint(0, 0))
		self.height = 10
		self.width = 60
		self.text = name

	def setSize(self, width, height):
		self.width = width
		self.height = height

	def setPinShapeFunc(self, shapeFunc):
		self.shapeFunc = shapeFunc

	def setType(self, type):
		self._type = type

	def setAcceptedType(self, type):
		self._acceptedTypes = type

	def itemChange(self, change, val):
		if change == 11:
			val.grabStarted.connect(self.grabStartedEvent)
			val.grabFinished.connect(self.grabFinished)
		return super(Pin, self).itemChange(change, val)

	def grabStartedEvent(self, pin):
		if pin == self:
			return
		if pin.isInput() and self.isOutput():
			if self._type in pin._acceptedTypes:
				self.overrideColor = QtCore.Qt.green
			else:
				self.overrideColor = QtCore.Qt.red
		elif pin.isOutput() and self.isInput():
			if pin._type in self._acceptedTypes and pin.node != self.node:
				self.overrideColor = QtCore.Qt.green
			else:
				self.overrideColor = QtCore.Qt.red
		self.update()

	def grabFinished(self):
		self.overrideColor = None

	def isConnected(self):
		return len(self.connection) > 0

	def setS(self, s):
		self.s = s

	def boundingRect(self):
		if self.pathShape is not None:
			return self.pathShape.boundingRect()
		boundingRect = QtCore.QRectF(-self.width / 2, -self.height / 2, self.width, self.height)
		return boundingRect

	def connectConnector(self, connection, emit=True):
		self.connection.append(connection)
		if self.isInput():
			if emit:
				self.onConnected.emit(connection)

	def disconnectConnector(self, connection, emit=True):
		if connection in self.connection:
			self.connection.remove(connection)
		if self.isInput():
			if emit:
				self.onDisconnected.emit(connection)

	def isOutput(self):
		return self._isInput == False

	def isInput(self):
		return self._isInput

	def mousePressEvent(self, event):
		if event.button() == QtCore.Qt.LeftButton:
			if self.boundingRect().contains(QtCore.QPoint(event.pos().x(), event.pos().y())):
				self.scene().connectionBuilder.prepareConnection(self)

	def paint(self, painter, option, widget):
		color = self.baseColor
		if (len(self.connection)):
			color = self.connectedColor
		if self.overrideColor is not None:
			color = self.overrideColor

		painter.setRenderHint(QtGui.QPainter.Antialiasing)
		painter.setRenderHint(QtGui.QPainter.SmoothPixmapTransform)

		brush = QtGui.QBrush(color)
		painter.setBrush(brush)
		painter.setPen(QtGui.QPen(QtGui.QColor(100, 100, 100), 1))

		if self.shapeFunc is not None:
			self.pathShape = self.shapeFunc(self, self.node)
		else:
			self.pathShape = QtGui.QPainterPath()

		painter.drawPath(self.pathShape)
		painter.fillPath(self.pathShape, color)
		# painter.drawRect(self.boundingRect())
		# NodeName

		'''brush = QBrush(QColor(0, 0, 0), Qt.SolidPattern)
		brush.setColor(QColor(0, 0, 0))
		painter.setBrush(brush)
		painter.setPen(QPen(Qt.white, 2, Qt.SolidLine, Qt.RoundCap, Qt.RoundJoin))
		painter.setFont(QFont("Times", 10, QFont.Bold))
		painter.drawText(QRectF(-self.size / 2, -7, 140, 20), self.name)'''

	###
	### Getter/Setter
	###
	def getNode(self):
		return self.node