from Ornatrix.QtImport import *
from PinPainter import *


class PinManager(object):
	def __init__(self, parent):
		self.parent = parent

		self._inputPins = {}
		self._outputPins = {}

	def addPin(self, pin):
		if pin.isInput():
			self._inputPins[pin.id] = pin
		else:
			self._outputPins[pin.id] = pin
		self.reorganisePin()

	def placePin(self, count, key, pin):
		nodeHeight = pin.parentItem().height / 2
		if pin.isOutput():
			nodeHeight = -nodeHeight
			nodeHeight += 5
		else:
			nodeHeight -= 5
		pin.prepareGeometryChange()
		if count == 1:
			pin.setPos(0, nodeHeight)
			pin.setSize(60, 10)
			pin.setPinShapeFunc(self.parent.defaultPainter.fullPinShape)
			pin.baseColor = QtGui.QColor(80, 80, 80)
			pin.connectedColor = QtGui.QColor(90, 90, 90)
		if count == 2:
			pin.setSize(15, 10)
			if key == 0:
				pin.baseColor = QtGui.QColor(0, 0, 150)
				pin.connectedColor = QtGui.QColor(0, 0, 170)
				pin.setPos(- 15, nodeHeight)
				pin.setPinShapeFunc(self.parent.defaultPainter.leftHalfPinShape)
			elif key == 1:
				pin.setPos(15, nodeHeight)
				pin.baseColor = QtGui.QColor(150, 0, 0)
				pin.connectedColor = QtGui.QColor(170, 0, 0)
				pin.setPinShapeFunc(self.parent.defaultPainter.RightHalfPinShape)

	def reorganisePin(self):
		inCount = len(self._inputPins)
		for key, pin in self._inputPins.iteritems():
			self.placePin(inCount, key, pin)
		outCount = len(self._outputPins)
		for key, pin in self._outputPins.iteritems():
			self.placePin(outCount, key, pin)

	###
	### Getter/Setter
	###

	def getAllConnector(self):
		connector = []
		for pin in self._inputPins.itervalues():
			connector.extend(pin.connection)
		for pin in self._outputPins.itervalues():
			connector.extend(pin.connection)
		return connector

	def getInputPin(self, id):
		if id in self._inputPins:
			return self._inputPins[id]

	def getOutputPin(self, id):
		if id in self._outputPins:
			return self._outputPins[id]

	# Return a list of node connected to the inputPin
	def getInputedNode(self):
		inputedNode = []
		for pin in self._inputPins.itervalues():
			if pin.isConnected():
				inputedNode.append(pin.connection[0].getSourcePin().parentItem())
		return inputedNode

	def hasConnectedInput(self):
		for pin in self._inputPins.itervalues():
			if pin.isConnected():
				return True
		return False

	def hasConnectedOutput(self):
		for pin in self._outputPins.itervalues():
			if pin.isConnected():
				return True
		return False
