from Ornatrix.QtImport import *

from Graph.GraphEditor import GraphEditor
from Stack.StackEditor import StackEditor
from .ControlMenu import ControlMenu

import Ornatrix.Global as oxgg


class OxGraphEditorUI(QtWidgets.QWidget):
	editorInst = None

	WINDOW_TITLE = "OxGraphEditor"

	def __init__(self, container, parent):
		super(OxGraphEditorUI, self).__init__(parent)

		self.container = container
		self.container.editor = self
		self.container.bridge.dataChangedEvent.connect(self.updateUI)
		oxgg.OxGraphEditor = self

		self.setWindowTitle("OxGraphEditor")
		# self.setWindowIcon(CreateIcon("frEditor.png"))
		# self.setGeometry(0, 0, 300, 300)

		self._layout = QtWidgets.QVBoxLayout()
		self._layout.setContentsMargins(0, 0, 0, 0)
		self.setLayout(self._layout)

		from ..styleSheet import setStyleSheet
		setStyleSheet(self)

		self.setSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Expanding)

		self.buildUI()
		self.updateUI()

	def buildUI(self):
		vbox = QtWidgets.QVBoxLayout()

		self.graphEditor = GraphEditor(self.container)
		self.graphEditor.hide()


		menuBar = QtWidgets.QMenuBar()

		fileMenu = QtWidgets.QMenu("File")

		stackMenu = QtWidgets.QMenu("Stack")
		stackMenu.addAction("Export Current Stack")

		graphMenu = QtWidgets.QMenu("Graph")
		stackMenu.addAction("Export Current HairSystem")
		autoLayout = QtWidgets.QAction("Automatic Layout", self)
		autoLayout.triggered.connect(self.graphEditor.oxScene.autoLayout)
		graphMenu.addAction(autoLayout)

		daisyChainAction = QtWidgets.QAction("Convert All Node To DaisyChain", self)
		daisyChainAction.triggered.connect(self.daisyChainConvert)
		fileMenu.addAction(daisyChainAction)

		helpMenu = QtWidgets.QMenu("Help")

		menuBar.addMenu(fileMenu)
		menuBar.addMenu(stackMenu)
		menuBar.addMenu(graphMenu)
		menuBar.addMenu(helpMenu)

		vbox.addWidget(menuBar)

		splitter = QtWidgets.QSplitter()
		###Top Menu

		left = QtWidgets.QWidget()
		left.setLayout(QtWidgets.QVBoxLayout())
		left.layout().addWidget(ControlMenu(self.container))

		self.stackEditor = StackEditor(self.container)
		left.layout().addWidget(self.stackEditor)

		###Bot Editor
		splitter.setSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Expanding)
		# Add Stack
		splitter.addWidget(left)

		# Add Graph
		splitter.addWidget(self.graphEditor)

		splitter.setCollapsible(0, False)
		splitter.setCollapsible(1, False)
		vbox.addWidget(splitter)
		###End Bottom Editor
		self._layout.addLayout(vbox)

	# Todo : find better place for that
	def daisyChainConvert(self):
		import Ornatrix.V2ToV2_5 as converter
		converter.convertStackToV25()

	def updateUI(self):
		self.container.bridge.startUndo(True)
		try:
			self.stackEditor.update()
		except Exception as e:
			print e
		try:
			self.graphEditor.updateUI()
		except Exception as e:
			print e
		self.container.bridge.endUndo(True)


	def displayGraph(self, open):
		self.graphEditor.show() if open else self.graphEditor.hide()

	def detachMe(self):
		self.setParent(None)
		self.deleteLater()
