from Ornatrix.QtImport import *
from ...Util.Enumeration import *

COLUMN = Enumeration('ICON STATE NAME')

from Ornatrix.Graph.Container.NodeContainer import *

class Branch(NodeContainer):
	def __init__(self, node):
		self.node = node
		self.uid = ""
		self.type = "branch"
		self.icon = None
		self.path = ""
		self._pathQty = 0
		self._pathChoice = 0
		self.rebuildName()

	@property
	def pathQty(self):
		return self._pathQty

	@property
	def pathChoice(self):
		return self._pathChoice

	@pathQty.setter
	def pathQty(self, pathQty):
		self._pathQty = pathQty
		self.rebuildName()

	@pathChoice.setter
	def pathChoice(self, pathChoice):
		self._pathChoice = pathChoice
		self.rebuildName()

	def rebuildName(self):
		self.name = " --- Branch --- "
		for r in range(0, self._pathQty):
			x = str(r)
			if self.pathChoice == r:
				x = " [" + str(r) + "] "
			self.name += x

	def supportState(self):
		return False

class treeItem(object):
	def __init__(self, node, parentItem):
		self.node = node
		self._parentItem = parentItem
		if self. _parentItem:
			self.depth = self._parentItem.depth + 1
		else:
			self.depth = 0

		self.childs = []

	def empty(self):
		for c in self.childs:
			c.empty()
		self.childs = []

	def appendChild(self, childItem):
		self.childs.insert(0, childItem)

	def childCount(self):
			return len(self.childs)

	def row(self):
		if self.parentItem:
			return self._parentItem.childs.index(self)
		return 0

	def parentItem(self):
		if self._parentItem:
			return self._parentItem

	def debug(self):
		if self._parentItem:
			print self._parentItem, self.node
		else:
			print self.node
		for c in self.childs:
			c.debug()


class StackModel(QtCore.QAbstractItemModel):
	iconActive_ = QtGui.QIcon(":icons/Operator_On.png")
	iconInactive_ = QtGui.QIcon(":icons/Operator_Off.png")
	iconShowEndResultOff_ = QtGui.QIcon(":icons/ShowEndResultOff.png")
	iconShowEndResultOn_ = QtGui.QIcon(":icons/ShowEndResultOn.png")

	stackRebuilded = QtCore.Signal()

	def __init__(self, container):
		super(StackModel, self).__init__()
		self.container = container
		self.container.registerUI("stackModel", self)

		self.container.uiRegisterCallback(self, "graph", self.setupGraphPlugin)

		self.test = None


		self.rootItem = treeItem(None, None)

		self.thumbSize = 64

		self.branchData = {}
		self.nodeList = []
		self.lastValidSelection = None

		self.update()

	###
	### UI Setup
	###
	def setupGraphPlugin(self, ui ,uiName, isDeregistering):
		if isDeregistering:
			ui.deregisterGraphPlugin("stack")
		else:
			from ..GraphPlugin.StackModelGraphPlugin import StackModelGraphPlugin
			graphPlugin = StackModelGraphPlugin(self)
			ui.registerGraphPlugin("stack", graphPlugin)
			self.stackRebuilded.connect(graphPlugin.rebuildGraph)



	###
	### Generation
	###

	def getOrCreateBranch(self, node):
		if node.uid not in self.branchData:
			self.branchData[node.uid] = Branch(node)
		return self.branchData[node.uid]

	def findPreferedPath(self, branch):
		# if last selected node is above branch,  we select that path in priority
		selectedOxNode = self.container.bridge.getSelectedOxNodes(True)
		if len(selectedOxNode) > 0:
			selNode = selectedOxNode[-1]
			aboveNodes = self.container.bridge.getNodeAbove(branch.node)
			for e in range(0, len(aboveNodes)):
				if self.container.bridge.isNodeAbove(selNode, aboveNodes[e]):
					branch.pathChoice = e
		# if the branch path choice is
		return branch.pathChoice

	def buildStack(self, startNode, parent):
		lowestNode = self.container.bridge.findLowestNode(startNode, True)
		if len(lowestNode) == 0:
			return

		currentNode = lowestNode[0]
		while (currentNode):
			item = treeItem(currentNode, parent)
			parent.appendChild(item)
			self.nodeList.append(currentNode)
			childs = currentNode.getChilds()
			if len(childs) > 0:
				self.buildStack(childs[0], item)

			nodeAboves = self.container.bridge.getNodeAbove(currentNode)
			result = []
			for res in nodeAboves:
				if res.getParent() != currentNode:
					result.append(res)
			pathChoice = 0
			if len(result) > 1:
				branch = self.getOrCreateBranch(currentNode)
				branch.pathQty = len(result)
				pathChoice = self.findPreferedPath(branch)
				branchItem = treeItem(branch, parent)
				parent.appendChild(branchItem)

			if len(result) >= 1 and result[0].getParent() == currentNode.getParent():
				currentNode = result[pathChoice]
			else:
				currentNode = None



	def update(self):
		self.beginResetModel()
		self.rootItem.empty()
		self.rootItem = treeItem(None, None)
		self.endResetModel()

		self.layoutAboutToBeChanged.emit()
		selectedOxNode = self.container.bridge.getSelectedOxNodes(True)

		if len(selectedOxNode) == 0:
			if self.lastValidSelection == None or self.lastValidSelection.isValid():
				return
		else:
			self.lastValidSelection = selectedOxNode[-1]
		self.selectedItem = []
		self.nodeList = []


		self.buildStack(self.lastValidSelection, self.rootItem)

		#self.rootItem.debug()
		self.layoutChanged.emit()
		self.stackRebuilded.emit()

	###
	### Virtual
	###

	def data(self, index, role=QtCore.Qt.DisplayRole):
		if not index.isValid():
			return
		item = index.internalPointer()
		node = item.node

		if role == QtCore.Qt.SizeHintRole:
			return QtCore.QSize(64, 32)
		if index.column() == COLUMN.ICON and node.icon:
			if role == QtCore.Qt.DecorationRole:
				return node.icon.scaled(QtCore.QSize(self.thumbSize, self.thumbSize), QtCore.Qt.KeepAspectRatio)
		if index.column() == COLUMN.STATE:
			if role == QtCore.Qt.DecorationRole:
				if index.row() == 0:
					return self.iconShowEndResultOn_ if node.enabled else self.iconShowEndResultOff_
				elif node.supportState():
					return self.iconActive_ if node.enabled else self.iconInactive_
		if index.column() == COLUMN.NAME:
			if role == QtCore.Qt.DisplayRole or role == QtCore.Qt.EditRole:
				return ("      " * (item.depth-1)) + node.name

		if role == QtCore.Qt.UserRole:
			return node


	def headerData(self, section, orientation, role):
		if role == QtCore.Qt.DisplayRole:
			if orientation == QtCore.Qt.Horizontal:
				if section == 1:
					return "On"
				elif section == 2:
					return "Operator"

	def flags(self, index):
		defaultFlag = super(StackModel, self).flags(index)
		if index.isValid():
			node = index.internalPointer().node
			if node.type == "branch":
				return QtCore.Qt.ItemIsEnabled
			elif node.config.getType() == "layer":
				return QtCore.Qt.ItemIsDropEnabled | defaultFlag
			else:
				return QtCore.Qt.ItemIsEditable | QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsDragEnabled | defaultFlag
		return QtCore.Qt.ItemIsDropEnabled | defaultFlag

	def iterate(self, index, func, depth = 0):
		if index.isValid():
			func(index, depth)
		if not self.hasChildren(index):
			return
		rows = self.rowCount(index)
		for row in range(0, rows):
			self.iterate(self.index(row, 0, index), func, depth+1)


	def index(self, row, column, parent):
		if not self.hasIndex(row, column, parent):
			return QtCore.QModelIndex()

		if not parent.isValid():
			parentItem = self.rootItem
		else:
			parentItem = parent.internalPointer()

		childItem = parentItem.childs[row]
		if childItem:
			return self.createIndex(row, column, childItem)
		else:
			return QtCore.QModelIndex()

	def parent(self, index):
		if not index.isValid():
			return

		childItem = index.internalPointer()
		parentItem = childItem.parentItem()


		if parentItem == self.rootItem:
			return QtCore.QModelIndex()
		return self.createIndex(parentItem.row(), 0, parentItem)


	def setData(self, index, value, role):
		if role != QtCore.Qt.EditRole:
			return False

		newName = value
		newName = newName.replace(' ', '_')
		self.container.bridge.renameNode(self.data(index, QtCore.Qt.UserRole), newName)
		return True

	def rowCount(self, parent):
		if parent.column() > 0:
			return 0
		if not parent.isValid():
			parentItem = self.rootItem
		else:
			parentItem = parent.internalPointer()
		return parentItem.childCount()

	def columnCount(self, parent):
		return 3


	###
	### Utility
	###

	def getFirstNodeAbove(self, node):
		aboveNodes = self.container.bridge.getNodeAbove(node)
		for aboveNode in aboveNodes:
			return aboveNode
		return None


	###
	### Action
	###


	def onItemClicked(self, index, selection=[]):
		self.layoutAboutToBeChanged.emit()
		node = index.internalPointer().node
		if index.column() == COLUMN.STATE:
			if node.supportState():
				self.container.bridge.toggleNode(node)
		self.layoutChanged.emit()

	def onItemDoubleClicked(self, index):
		node = index.internalPointer().node
		if node.type == "branch":
			node.pathChoice += 1
			if node.pathChoice >= node.pathQty:
				node.pathChoice = 0
			self.update()
			return True
		return False


	###
	### Drop
	###

	def supportedDropActions(self):
		return QtCore.Qt.CopyAction | QtCore.Qt.MoveAction

	def mimeTypes(self):
		return ["application/ox-graph-array"]

	def mimeData(self, indexes):
		data = QtCore.QMimeData()
		text = ""
		for index in indexes:
			if len(text) > 0:
				text += "|"
			text += str(index.row())

		list = []
		for index in indexes:
			list.append(index.internalPointer().node.path)
		list = set(list)
		names = '\n'.join(list)

		data.setText(names)
		data.setData('application/ox-graph-array', text)
		return data

	def dropMimeData(self, data, action, rowIndex, column, parent):
		nodes = data.text().split("\n")
		nodes.reverse()

		print rowIndex, parent.isValid()
		target = self.index(rowIndex, column, parent)
		targetNode = target.internalPointer().node

		aboveBranch = False

		if targetNode.name == "Branch":
			targetNode = self.index(rowIndex + 1, column, parent)
			aboveBranch = True

		nodeToReorder = []
		for nodeName in nodes:
			print "reordering ", nodeName
			node = self.container.bridge.wrapNode(nodeName)
			aboveNodes = self.container.bridge.getNodeAbove(node)
			validAboveNode = self.getFirstNodeAbove(node)
			underNode = self.container.bridge.getNodeUnder(node)[0]
			self.container.bridge.disconnectNode(node, validAboveNode)
			self.container.bridge.disconnectNode(underNode, node)
			for aboveNode in aboveNodes:
				self.container.bridge.connectNode(underNode, aboveNode)
			nodeToReorder.append(node)

		# We gather the new node that will serve as final node
		if aboveBranch:
			finalGoals = [self.getFirstNodeAbove(targetNode)]
		else:
			finalGoals = self.container.bridge.getNodeAbove(targetNode)
		print targetNode
		print finalGoals

		# We reconnect each node in the list one by one, using the previous node as the target Node

		if parent.isValid():
			parentNode = parent.internalPointer().node
		else:
			parentNode = None

		for node in nodeToReorder:
			self.container.bridge.connectNode(targetNode, node)
			self.container.bridge.setParent(node, parentNode)
			targetNode = node

		for finalGoal in finalGoals:
			self.container.bridge.connectNode(targetNode, finalGoal)


		self.update()

		return True

	###
	### Query
	###
	def nodeInStack(self, node):
		if node in self.nodeList:
			return True
		return False