from Ornatrix.QtImport import *
from Ornatrix.Graph.Container.HostNode import *

import TableViewEventFilter


class StackView(QtWidgets.QTreeView):

	def __init__(self, container):
		super(StackView, self).__init__()

		self.container = container


		self.copiedNodes = []

		self.lockSelect = False
		sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)

		sizePolicy.setHorizontalStretch(0)
		sizePolicy.setVerticalStretch(0)
		sizePolicy.setHeightForWidth(self.sizePolicy().hasHeightForWidth())
		self.setMinimumWidth(200)
		self.setSizePolicy(sizePolicy)
		self.setEditTriggers(
			QtWidgets.QAbstractItemView.DoubleClicked |
			QtWidgets.QAbstractItemView.EditKeyPressed |
			QtWidgets.QAbstractItemView.SelectedClicked)
		self.setWordWrap(False)

		# Header Setup
		# header = self.horizontalHeader()
		# header.setStretchLastSection(True)
		# header.setSectionsClickable(False)
		# header.setSectionResizeMode(QtWidgets.QHeaderView.Fixed)


		self.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
		self.setDragDropMode(QtWidgets.QAbstractItemView.InternalMove);
		self.setAutoExpandDelay(1)
		self.setDragEnabled(True)
		self.viewport().setAcceptDrops(True)
		self.setDropIndicatorShown(True)


		self.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
		self.customContextMenuRequested.connect(self.customMenuRequested)

		# self.verticalHeader().setVisible(False)



		self.clicked.connect(self.onClick)

	def update(self):
		self.lockSelect = True
		self.model().iterate(self.rootIndex(), self.checkSelected)
		self.lockSelect = False



	def checkSelected(self, index, depth):
		n = index.internalPointer().node
		if isinstance(n, HostNode) and n.selected:
			self.selectionModel().select(index, QtCore.QItemSelectionModel.Select | QtCore.QItemSelectionModel.Rows)

	def showEvent(self, event):
		self.setColumnWidth(0, 64)

	def customMenuRequested(self, pos):
		index = self.indexAt(pos)

		menu = QtWidgets.QMenu()
		nodes = []
		hasShapeSelected = False
		for selIndex in self.getSelectedRows():
			node = self.model().data(selIndex, QtCore.Qt.UserRole)
			if node.isBaseShape():
				return
			nodes.append(node)
			if node.isTopShape():
				hasShapeSelected = True

		self.actionSelection_ = nodes

		if not hasShapeSelected:
			action = QtWidgets.QAction('Copy', self)
			action.triggered.connect(self.copyNode)
			menu.addAction(action)

			action = QtWidgets.QAction('Paste', self)
			action.triggered.connect(self.pasteNode)
			menu.addAction(action)

		menu.addAction('delete')
		menu.exec_(self.viewport().mapToGlobal(pos))

	def onClick(self, index):
		self.model().onItemClicked(index)

	def selectionChanged(self, selected, deselected):
		super(StackView, self).selectionChanged(selected, deselected)
		if not self.lockSelect:
			newSelect = []
			for sel in self.selectionModel().selectedIndexes():
				item = sel.internalPointer()
				node = item.node
				if isinstance(node, HostNode):
					newSelect.append(node.path)
			self.container.bridge.setSelection(newSelect)


	def edit(self, index, trigger, event):
		if trigger == QtWidgets.QAbstractItemView.DoubleClicked:
			if self.model().onItemDoubleClicked(index):
				return False
		return super(StackView, self).edit(index, trigger, event)

	def getSelectedRows(self):
		selections = self.selectionModel()
		selected = selections.selectedIndexes()
		return selected

	def copyNode(self):
		self.copiedNodes = reversed(self.actionSelection_)

	def pasteNode(self):
		if len(self.actionSelection_) != 1:
			return

		validNodes = [node for node in self.copiedNodes if node.isValid()]
		if len(validNodes) == 0:
			return

		lastNode = self.actionSelection_[0]
		targets = self.container.bridge.getNodeAbove(lastNode)
		for node in validNodes:
			copied = self.container.bridge.copyNode(node)
			self.container.bridge.connectNode(lastNode, copied)
			for target in targets:
				self.container.bridge.connectNode(copied, target)
			lastNode = copied
