from shiboken2 import wrapInstance

import maya.cmds as mc
import maya.mel as mel
import maya.OpenMayaUI as mui

from Ornatrix import Global as oxgg
from Graph.Editor.OxGraphEditorUI import OxGraphEditorUI

from MayaBridge import *
from Graph.Container.Container import *

isInitiliazed = False
GraphName = "OxGraph001"


def devReload():
	pythonCleanup()

	import gc
	gc.collect()

	import sys
	for item in sorted(sys.modules.keys()):
		if 'Ornatrix' in item:
			del (sys.modules[item])


def forceRefresh():
	oxgg.OxGraphEditor.editor.bridge.dataChangedEvent.emit()


'''
Called when the plugin is deregistered
'''


def pythonCleanup():
	if mc.workspaceControl(GraphName, q=True, ex=True):
		mc.workspaceControl(GraphName, e=True, retain=False)
		mc.workspaceControl(GraphName, e=True, close=True)

	if oxgg.OxGraphEditor is not None:
		oxgg.OxGraphEditor.detachMe()
		oxgg.OxGraphEditor = None


def createOrnatrixEditor(showIt=True):
	checkPluginStatus()
	if showIt:
		if mc.workspaceControl(GraphName, q=True, ex=True):
			mc.workspaceControl(GraphName, e=True, vis=True, r=True)
		else:
			LEcomponent = mel.eval('getUIComponentDockControl("Channel Box / Layer Editor", false);')
			uiScript = r'''import Ornatrix.MayaInitializer as OxGraph
OxGraph.createDockControl()'''
			mc.workspaceControl(GraphName,
			                    ttc=(LEcomponent, -1),
			                    retain=False,
			                    uiScript=uiScript,
			                    mw=400,
			                    label='Ornatrix Editor')


def closeDockControl():
	if oxgg.OxGraphEditor:
		oxgg.OxGraphEditor.detachMe()
		oxgg.OxGraphEditor = None


def createDockControl():
	checkPluginStatus()
	if oxgg.Maya:
		parent = mui.MQtUtil.getCurrentParent()
		parentWidget = wrapInstance(long(parent), QtWidgets.QWidget)
		# widget = mui.MQtUtil.findControl(GraphName)

		if oxgg.OxGraphEditor is None:
			container = OrnatrixContainer()
			container.bridge = MayaBridge()
			OxGraphEditorUI(container, parentWidget)
			oxgg.OxGraphEditor.setObjectName(GraphName)

		parentWidget.layout().addWidget(oxgg.OxGraphEditor)
		oxgg.OxGraphEditor.updateUI()

		if parentWidget:
			parentWidget.destroyed.connect(oxgg.OxGraphEditor.detachMe)


def checkPluginStatus():
	if (mc.pluginInfo('Ornatrix', query=True, loaded=True) is False):
		mc.loadPlugin('Ornatrix')
