import maya.cmds as mc

from Ornatrix.QtImport import *

import maya.OpenMayaUI as mui


class channelData():
	def __init__(self, name):
		self.name = name
		self.size = 1


class channelCreator(QtWidgets.QDialog):
	def __init__(self, container, name="", type=0, parent=None):
		super(channelCreator, self).__init__(parent)

		self.container = container
		self.name = name

		self.edit = False
		self.editIndice = -1

		vbox = QtWidgets.QVBoxLayout()

		# Name and Type
		hbox = QtWidgets.QHBoxLayout()

		self.newName = QtWidgets.QLineEdit()
		self.newName.setText(self.name)

		hbox.addWidget(QtWidgets.QLabel('name : '))
		hbox.addWidget(self.newName)
		vbox.addLayout(hbox)

		# Tree View
		hbox = QtWidgets.QHBoxLayout()

		self.leftModel = QtCore.QStringListModel()
		self.leftView = QtWidgets.QListView()
		self.leftView.setModel(self.leftModel)

		centerControl = QtWidgets.QVBoxLayout()

		self.add = QtWidgets.QPushButton(">>")
		self.add.clicked.connect(self.addChannel)
		self.remove = QtWidgets.QPushButton("<<")
		self.remove.clicked.connect(self.removeChannel)
		centerControl.addWidget(self.add)
		centerControl.addWidget(self.remove)

		self.rightModel = QtCore.QStringListModel()
		self.rightView = QtWidgets.QListView()
		self.rightView.setModel(self.rightModel)

		hbox.addWidget(self.leftView)
		hbox.addLayout(centerControl)
		hbox.addWidget(self.rightView)
		vbox.addLayout(hbox)

		# TreeViewEnd

		# Comfirm
		self.accept = QtWidgets.QPushButton("Confirm")
		self.accept.clicked.connect(self.finish)

		vbox.addWidget(self.accept)

		self.setLayout(vbox)

		self.channelList = []

		channels = mc.OxGetRootChannels(self.container.inputAttr)
		for i in range(len(channels)):
			self.channelList.append([i, channels[i]])

		self.addedList = []

		self.refresh()

	def setEdit(self, indice):
		self.edit = True
		self.editIndice = indice

	def finish(self):
		name = self.newName.text()
		if len(name) == 0:
			noName = QtWidgets.QMessageBox(self)
			noName.setText("You must enter a name")
			noName.setIcon(QtWidgets.QMessageBox.Warning)
			noName.exec_()
			return

		indice = mc.getAttr(self.container.namesAttr, size=True)
		if self.edit:
			indice = self.editIndice

		list = []
		for l in self.addedList:
			list.append(l[0])
		mc.setAttr(self.container.namesAttr + "[" + str(indice) + "]", self.name, type="string")
		mc.setAttr(self.container.idsAttr + "[" + str(indice) + "]", list, type="Int32Array")
		self.container.refresh()
		self.close()

	def addChannel(self):
		for i in self.leftView.selectedIndexes():
			chann = self.channelList[i.row()]
			self.addedList.append(chann)
			self.channelList.pop(i.row())
		self.refresh()

	def removeChannel(self):
		for i in self.rightView.selectedIndexes():
			chann = self.addedList[i.row()]
			self.channelList.append(chann)
			self.removeChannel().pop(i.row())
		self.refresh()

	def refresh(self):
		stringList = []
		for chann in self.channelList:
			stringList.append(chann[1])
		self.leftModel.setStringList(stringList)
		stringList = []
		for chann in self.addedList:
			stringList.append(chann[1])
		self.rightModel.setStringList(stringList)


class addAttrWidget(QtWidgets.QWidget):
	def __init__(self, container):
		super(addAttrWidget, self).__init__()

		self.container = container

		self._layout = QtWidgets.QHBoxLayout()

		self.newName = QtWidgets.QLineEdit()
		self.type = QtWidgets.QComboBox()
		self.type.addItem("scalar")
		self.type.addItem("color")
		self.addButton = QtWidgets.QPushButton("+")
		self.addButton.clicked.connect(self.displayChannelCreator)

		self._layout.addWidget(QtWidgets.QLabel('name : '))
		self._layout.addWidget(self.newName)
		self._layout.addWidget(QtWidgets.QLabel('type : '))
		self._layout.addWidget(self.type)
		self._layout.addWidget(self.addButton)

		self.setLayout(self._layout)

	def displayChannelCreator(self):
		creator = channelCreator(self.container, self.newName.text(), self.type.currentIndex())
		creator.exec_()


class existingChannel(QtWidgets.QWidget):
	def __init__(self, container, indice, name, idx):
		super(existingChannel, self).__init__()
		self.container = container
		self.indice = indice
		self.name = name
		self.idx = idx

		self.layout = QtWidgets.QHBoxLayout()
		self.layout.addWidget(QtWidgets.QLabel(name))

		channels = self.getExportedChannel()
		str = "["
		for channel in channels:
			if len(str) > 2:
				str += " | "
			str += channel
		str += "]"
		self.layout.addWidget(QtWidgets.QLabel(str))

		self.edit = QtWidgets.QPushButton('Edit')
		self.edit.clicked.connect(self.editMe)
		self.delete = QtWidgets.QPushButton("X")
		self.delete.clicked.connect(self.deleteMe)
		self.layout.addWidget(self.edit)
		self.layout.addWidget(self.delete)

		self.setLayout(self.layout)

	def editMe(self):
		creator = channelCreator(self.container, self.name, 0)
		creator.setEdit(self.indice)
		creator.exec_()

	def deleteMe(self):
		mc.removeMultiInstance(self.container.namesAttr + "[" + str(self.indice) + "]")
		mc.removeMultiInstance(self.container.idsAttr + "[" + str(self.indice) + "]")

	def getExportedChannel(self):
		found = []
		channels = mc.OxGetRootChannels(self.container.inputAttr)
		for i in self.idx:
			if i < len(channels):
				found.append(channels[i])
			else:
				found.append('__deleted__')
		return found


class AERendererExport(QtWidgets.QWidget):
	def __init__(self, inputAttr, namesAttr, idsAttr, parent=None):
		super(AERendererExport, self).__init__(parent)
		self.inputAttr = inputAttr
		self.namesAttr = namesAttr
		self.idsAttr = idsAttr

		self._layout = QtWidgets.QVBoxLayout(self)
		self._layout.setContentsMargins(5, 3, 11, 3)
		self._layout.setSpacing(5)

		self.setLayout(self._layout)

		self.refresh()

	def setAttributePath(self, inputAttr, namesAttr, idsAttr):
		self.inputAttr = inputAttr
		self.namesAttr = namesAttr
		self.idsAttr = idsAttr

	def refresh(self):
		clearLayout(self._layout)

		indices = mc.getAttr(self.namesAttr, mi=True)
		if indices:
			for indice in indices:
				name = mc.getAttr(self.namesAttr + "[" + str(indice) + "]")
				channel = mc.getAttr(self.idsAttr + "[" + str(indice) + "]")
				ex = existingChannel(self, indice, name, channel)
				self._layout.addWidget(ex)

		adder = addAttrWidget(self)
		self._layout.addWidget(adder)


def buildQT(lay, inputAttr, namesAttr, idsAttr):
	# get the layout object
	widget = getMayaWidget(lay)
	# create the GUI/widget
	widg = AERendererExport(inputAttr, namesAttr, idsAttr)

	# add the widget to the layout
	widget.layout().addWidget(widg)


def updateQT(lay, inputAttr, namesAttr, idsAttr):
	# get the layout
	widget = getMayaWidget(lay)
	layout = widget.layout()

	# find the widget
	for c in range(layout.count()):
		widg = layout.itemAt(c).widget()
		if isinstance(widg, AERendererExport):
			# found the widget, update the node it's pointing to
			widg.setAttributePath(inputAttr, namesAttr, idsAttr)
			widg.refresh()
			break
