import os

from rfm2.config import cfg
from rfm2.extensions import _add_extension_attr_float, _add_extension_attr_string
from rfm2.render import rman_cmd
from rfm2.utils.node_desc import NodeDesc, NodeDescParamJSON
from rfm2.utils.filepath import FilePath
from rfm2.utils import json_file

from maya.api import OpenMaya


def register_ornatrix():
    """Register translation information for Ornatrix HairShape
    
    This is hopefully temporary until they add a way to specify plugin extension path env
    """
    type_name = u'HairShape'
    proc_name = u'OxHairPRManProcedural'
    extension_dir = os.getenv( 'PRMAN_EXTENSIONS_PATH' )

    # add to procedural search path
    cfg().rman_search_paths['procedural'].append( extension_dir )

    # add required extension attributes for the procedural
    data = json_file.load( os.path.sep.join( [extension_dir, 'OxHairPRManExtension.json'] ) )
    hair_shape_class = OpenMaya.MNodeClass( type_name )

    cfg().extensions[type_name] = []
    cfg().extensionParams[type_name] = {}

    for attr_dict in data['attributes']:
        param = NodeDescParamJSON( attr_dict )
        cfg().extensions[type_name].append( param )
        cfg().extensionParams[type_name][attr_dict['name']] = param

        if param.type == 'string':
            _add_extension_attr_string( hair_shape_class, attr_dict )
        elif param.type == 'float':
            _add_extension_attr_float( hair_shape_class, attr_dict )

    # register procedural translator
    ox_proc_path = FilePath( os.path.sep.join( [extension_dir, proc_name] ) + '.args' )
    ox_proc_desc = NodeDesc( ox_proc_path )

    cfg().translationTable.extend( [type_name, proc_name] )
    cfg().translatePxrToMayaDict[proc_name] = type_name
    cfg().translateMayaToPxrDict[type_name] = proc_name

    cfg().register_node_description( ox_proc_desc )

    rman_cmd( ( 'nodeTable ' + ' '.join( cfg().translationTable ) ).encode( 'ascii' ) )
